@@ -143,10 +143,10 @@ parseForecastXml (const char *buff, WeatherInfo *master_info)
                                 xmlChar *val = xmlNodeGetContent (c);
 
                                 if (val) {
-                                    GTimeVal tv;
-
-                                    if (g_time_val_from_iso8601 ((const char *)val, &tv)) {
-                                        update_times[count] = tv.tv_sec;
+                                    GDateTime *dt = g_date_time_new_from_iso8601 ((const char *)val, NULL);
+                                    if (dt != NULL) {
+                                        update_times[count] = g_date_time_to_unix (dt);
+                                        g_date_time_unref (dt);
                                     } else {
                                         update_times[count] = 0;
                                     }
@@ -422,18 +422,20 @@ iwin_start_open (WeatherInfo *info)
         info->forecast = NULL;
     }
 
-    free_forecast_list (info);    
+    free_forecast_list (info);
 
     if (info->forecast_type == FORECAST_LIST) {
         /* see the description here: http://www.weather.gov/forecasts/xml/ */
         if (loc->latlon_valid) {
-            struct tm tm;
-            time_t now = time (NULL);
+            GDateTime *dt;
+            gint year, month, day;
 
-            localtime_r (&now, &tm);
+            dt = g_date_time_new_now_local ();
+            g_date_time_get_ymd (dt, &year, &month, &day);
+            g_date_time_unref (dt);
 
             url = g_strdup_printf ("http://www.weather.gov/forecasts/xml/sample_products/browser_interface/ndfdBrowserClientByDay.php?&lat=%.02f&lon=%.02f&format=24+hourly&startDate=%04d-%02d-%02d&numDays=7",
-                       RADIANS_TO_DEGREES (loc->latitude), RADIANS_TO_DEGREES (loc->longitude), 1900 + tm.tm_year, 1 + tm.tm_mon, tm.tm_mday);
+                       RADIANS_TO_DEGREES (loc->latitude), RADIANS_TO_DEGREES (loc->longitude), year, month, day);
 
             msg = soup_message_new ("GET", url);
             g_free (url);
@@ -441,7 +443,6 @@ iwin_start_open (WeatherInfo *info)
 
             info->requests_pending++;
         }
-
         return;
     }
 
@@ -462,11 +463,11 @@ iwin_start_open (WeatherInfo *info)
     zone = g_ascii_strdown (loc->zone, -1);
     state = g_strndup (zone, 2);
 
-    url = g_strdup_printf ("http://weather.noaa.gov/pub/data/forecasts/zone/%s/%s.txt", state, zone);
+    url = g_strdup_printf ("http://tgftp.nws.noaa.gov/data/forecasts/zone/%s/%s.txt", state, zone);
 
     g_free (zone);
     g_free (state);
-    
+
     msg = soup_message_new ("GET", url);
     g_free (url);
     soup_session_queue_message (info->session, msg, iwin_finish, info);
