@@ -26,6 +26,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
+#include <fenv.h>
 
 #ifdef HAVE_VALUES_H
 #include <values.h>
@@ -42,11 +43,6 @@
 
 #define MOON_PHASES 36
 
-/* g_memmove is removed in glib 2.40 */
-#if GLIB_CHECK_VERSION (2, 39, 0) && !defined(g_memmove)
-#define g_memmove memmove
-#endif
-
 /**
  * SECTION:weather
  * @Title: weather
@@ -295,7 +291,7 @@ weather_sky_string (WeatherSky sky)
 /*                   NONE                         VICINITY                             LIGHT                      MODERATE                      HEAVY                      SHALLOW                      PATCHES                         PARTIAL                      THUNDERSTORM                    BLOWING                      SHOWERS                         DRIFTING                      FREEZING                      */
 /*               *******************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************/
 static const gchar *conditions_str[24][13] = {
-/* TRANSLATOR: If you want to know what "blowing" "shallow" "partial"
+/* Translators: If you want to know what "blowing" "shallow" "partial"
  * etc means, you can go to http://www.weather.com/glossary/ and
  * http://www.crh.noaa.gov/arx/wx.tbl.php */
     /* NONE          */ {"??",                        "??",                                "??",                      "??",                         "??",                      "??",                        "??",                           "??",                        N_("Thunderstorm"),             "??",                        "??",                           "??",                         "??"                         },
@@ -557,8 +553,7 @@ _weather_info_fill (WeatherInfo *info,
     info->cb_data = data;
 
     if (!info->session) {
-	info->session = soup_session_new_with_options (SOUP_SESSION_TIMEOUT, MATEWEATHER_DEFAULT_TIMEOUT, NULL);
-	soup_session_add_feature_by_type (info->session, SOUP_TYPE_PROXY_RESOLVER_DEFAULT);
+        info->session = soup_session_new ();
     }
 
     metar_start_open (info);
@@ -579,7 +574,6 @@ weather_info_abort (WeatherInfo *info)
     if (info->session) {
 	soup_session_abort (info->session);
 	info->requests_pending = 0;
-	info->session = NULL;
     }
 }
 
@@ -594,7 +588,7 @@ weather_info_clone (const WeatherInfo *info)
 
 
     /* move everything */
-    g_memmove (clone, info, sizeof (WeatherInfo));
+    memmove (clone, info, sizeof (WeatherInfo));
 
 
     /* special moves */
@@ -643,7 +637,7 @@ weather_info_free (WeatherInfo *info)
         g_object_unref (info->radar);
         info->radar = NULL;
     }
-	
+
     g_free (info);
 }
 
@@ -712,7 +706,7 @@ weather_info_get_update (WeatherInfo *info)
     if (info->update != 0) {
         struct tm tm;
         localtime_r (&info->update, &tm);
-	/* TRANSLATOR: this is a format string for strftime
+	/* Translators: this is a format string for strftime
 	 *             see `man 3 strftime` for more details
 	 */
 	timeformat = g_locale_from_utf8 (_("%a, %b %d / %H:%M"), -1,
@@ -758,36 +752,60 @@ weather_info_get_conditions (WeatherInfo *info)
 }
 
 static const gchar *
-temperature_string (gfloat temp_f, TempUnit to_unit, gboolean want_round)
+temperature_string (gdouble temp, TempUnit to_unit, gboolean want_round)
 {
     static gchar buf[100];
 
     switch (to_unit) {
     case TEMP_UNIT_FAHRENHEIT:
 	if (!want_round) {
-	    /* TRANSLATOR: This is the temperature in degrees Fahrenheit (\302\260 is U+00B0 DEGREE SIGN) */
-	    g_snprintf (buf, sizeof (buf), _("%.1f \302\260F"), temp_f);
+	    /* Translators: This is the temperature in degrees Fahrenheit (\302\260 is U+00B0 DEGREE SIGN) */
+	    g_snprintf (buf, sizeof (buf), _("%.1f \302\260F"), temp);
 	} else {
-	    /* TRANSLATOR: This is the temperature in degrees Fahrenheit (\302\260 is U+00B0 DEGREE SIGN) */
-	    g_snprintf (buf, sizeof (buf), _("%d \302\260F"), (int)floor (temp_f + 0.5));
+	    const int range_problem = FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW | FE_UNDERFLOW;
+	    gdouble temp_r;
+
+	    feclearexcept(range_problem);
+	    temp_r = round (temp);
+	    if (fetestexcept(range_problem))
+	        g_snprintf (buf, sizeof (buf), _("n/a"));
+	    else
+	        /* Translators: This is the temperature in degrees Fahrenheit (\302\260 is U+00B0 DEGREE SIGN) */
+	        g_snprintf (buf, sizeof (buf), _("%d \302\260F"), (int)temp_r);
 	}
 	break;
     case TEMP_UNIT_CENTIGRADE:
 	if (!want_round) {
-	    /* TRANSLATOR: This is the temperature in degrees Celsius (\302\260 is U+00B0 DEGREE SIGN) */
-	    g_snprintf (buf, sizeof (buf), _("%.1f \302\260C"), TEMP_F_TO_C (temp_f));
+	    /* Translators: This is the temperature in degrees Celsius (\302\260 is U+00B0 DEGREE SIGN) */
+	    g_snprintf (buf, sizeof (buf), _("%.1f \302\260C"), TEMP_F_TO_C (temp));
 	} else {
-	    /* TRANSLATOR: This is the temperature in degrees Celsius (\302\260 is U+00B0 DEGREE SIGN) */
-	    g_snprintf (buf, sizeof (buf), _("%d \302\260C"), (int)floor (TEMP_F_TO_C (temp_f) + 0.5));
+	    const int range_problem = FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW | FE_UNDERFLOW;
+	    gdouble temp_r;
+
+	    feclearexcept(range_problem);
+	    temp_r = round (TEMP_F_TO_C (temp));
+	    if (fetestexcept(range_problem))
+	        g_snprintf (buf, sizeof (buf), _("n/a"));
+	    else
+	        /* Translators: This is the temperature in degrees Celsius (\302\260 is U+00B0 DEGREE SIGN) */
+	        g_snprintf (buf, sizeof (buf), _("%d \302\260C"), (int)temp_r);
 	}
 	break;
     case TEMP_UNIT_KELVIN:
 	if (!want_round) {
-	    /* TRANSLATOR: This is the temperature in kelvin */
-	    g_snprintf (buf, sizeof (buf), _("%.1f K"), TEMP_F_TO_K (temp_f));
+	    /* Translators: This is the temperature in kelvin */
+	    g_snprintf (buf, sizeof (buf), _("%.1f K"), TEMP_F_TO_K (temp));
 	} else {
-	    /* TRANSLATOR: This is the temperature in kelvin */
-	    g_snprintf (buf, sizeof (buf), _("%d K"), (int)floor (TEMP_F_TO_K (temp_f)));
+	    const int range_problem = FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW | FE_UNDERFLOW;
+	    gdouble temp_r;
+
+	    feclearexcept(range_problem);
+	    temp_r = round (TEMP_F_TO_K (temp));
+	    if (fetestexcept(range_problem))
+	        g_snprintf (buf, sizeof (buf), _("n/a"));
+	    else
+	        /* Translators: This is the temperature in kelvin */
+	        g_snprintf (buf, sizeof (buf), _("%d K"), (int)temp_r);
 	}
 	break;
 
@@ -868,7 +886,7 @@ weather_info_get_humidity (WeatherInfo *info)
     if (humidity < 0.0)
         return _("Unknown");
 
-    /* TRANSLATOR: This is the humidity in percent */
+    /* Translators: This is the humidity in percent */
     g_snprintf (buf, sizeof (buf), _("%.f%%"), humidity);
     return buf;
 }
@@ -896,23 +914,23 @@ windspeed_string (gfloat knots, SpeedUnit to_unit)
 
     switch (to_unit) {
     case SPEED_UNIT_KNOTS:
-	/* TRANSLATOR: This is the wind speed in knots */
+	/* Translators: This is the wind speed in knots */
 	g_snprintf (buf, sizeof (buf), _("%0.1f knots"), knots);
 	break;
     case SPEED_UNIT_MPH:
-	/* TRANSLATOR: This is the wind speed in miles per hour */
+	/* Translators: This is the wind speed in miles per hour */
 	g_snprintf (buf, sizeof (buf), _("%.1f mph"), WINDSPEED_KNOTS_TO_MPH (knots));
 	break;
     case SPEED_UNIT_KPH:
-	/* TRANSLATOR: This is the wind speed in kilometers per hour */
+	/* Translators: This is the wind speed in kilometers per hour */
 	g_snprintf (buf, sizeof (buf), _("%.1f km/h"), WINDSPEED_KNOTS_TO_KPH (knots));
 	break;
     case SPEED_UNIT_MS:
-	/* TRANSLATOR: This is the wind speed in meters per second */
+	/* Translators: This is the wind speed in meters per second */
 	g_snprintf (buf, sizeof (buf), _("%.1f m/s"), WINDSPEED_KNOTS_TO_MS (knots));
 	break;
     case SPEED_UNIT_BFT:
-	/* TRANSLATOR: This is the wind speed as a Beaufort force factor
+	/* Translators: This is the wind speed as a Beaufort force factor
 	 * (commonly used in nautical wind estimation).
 	 */
 	g_snprintf (buf, sizeof (buf), _("Beaufort force %.1f"),
@@ -943,7 +961,7 @@ weather_info_get_wind (WeatherInfo *info)
         strncpy (buf, _("Calm"), sizeof (buf));
 	buf[sizeof (buf)-1] = '\0';
     } else {
-        /* TRANSLATOR: This is 'wind direction' / 'wind speed' */
+        /* Translators: This is 'wind direction' / 'wind speed' */
         g_snprintf (buf, sizeof (buf), _("%s / %s"),
 		    weather_wind_direction_string (info->wind),
 		    windspeed_string (info->windspeed, info->speed_unit));
@@ -965,27 +983,27 @@ weather_info_get_pressure (WeatherInfo *info)
 
     switch (info->pressure_unit) {
     case PRESSURE_UNIT_INCH_HG:
-	/* TRANSLATOR: This is pressure in inches of mercury */
+	/* Translators: This is pressure in inches of mercury */
 	g_snprintf (buf, sizeof (buf), _("%.2f inHg"), info->pressure);
 	break;
     case PRESSURE_UNIT_MM_HG:
-	/* TRANSLATOR: This is pressure in millimeters of mercury */
+	/* Translators: This is pressure in millimeters of mercury */
 	g_snprintf (buf, sizeof (buf), _("%.1f mmHg"), PRESSURE_INCH_TO_MM (info->pressure));
 	break;
     case PRESSURE_UNIT_KPA:
-	/* TRANSLATOR: This is pressure in kiloPascals */
+	/* Translators: This is pressure in kiloPascals */
 	g_snprintf (buf, sizeof (buf), _("%.2f kPa"), PRESSURE_INCH_TO_KPA (info->pressure));
 	break;
     case PRESSURE_UNIT_HPA:
-	/* TRANSLATOR: This is pressure in hectoPascals */
+	/* Translators: This is pressure in hectoPascals */
 	g_snprintf (buf, sizeof (buf), _("%.2f hPa"), PRESSURE_INCH_TO_HPA (info->pressure));
 	break;
     case PRESSURE_UNIT_MB:
-	/* TRANSLATOR: This is pressure in millibars */
+	/* Translators: This is pressure in millibars */
 	g_snprintf (buf, sizeof (buf), _("%.2f mb"), PRESSURE_INCH_TO_MB (info->pressure));
 	break;
     case PRESSURE_UNIT_ATM:
-	/* TRANSLATOR: This is pressure in atmospheres */
+	/* Translators: This is pressure in atmospheres */
 	g_snprintf (buf, sizeof (buf), _("%.3f atm"), PRESSURE_INCH_TO_ATM (info->pressure));
 	break;
 
@@ -1013,15 +1031,15 @@ weather_info_get_visibility (WeatherInfo *info)
 
     switch (info->distance_unit) {
     case DISTANCE_UNIT_MILES:
-	/* TRANSLATOR: This is the visibility in miles */
+	/* Translators: This is the visibility in miles */
 	g_snprintf (buf, sizeof (buf), _("%.1f miles"), info->visibility);
 	break;
     case DISTANCE_UNIT_KM:
-	/* TRANSLATOR: This is the visibility in kilometers */
+	/* Translators: This is the visibility in kilometers */
 	g_snprintf (buf, sizeof (buf), _("%.1f km"), VISIBILITY_SM_TO_KM (info->visibility));
 	break;
     case DISTANCE_UNIT_METERS:
-	/* TRANSLATOR: This is the visibility in meters */
+	/* Translators: This is the visibility in meters */
 	g_snprintf (buf, sizeof (buf), _("%.0fm"), VISIBILITY_SM_TO_M (info->visibility));
 	break;
 
@@ -1657,41 +1675,3 @@ _weather_internal_check (void)
     g_assert (G_N_ELEMENTS (conditions_str) == PHENOMENON_LAST);
     g_assert (G_N_ELEMENTS (conditions_str[0]) == QUALIFIER_LAST);
 }
-
-/**
- * weather_info_equal:
- * @info1:  WeatherInfo to compare
- * @info2:  WeatherInfo to compare
- *
- * Returns: gboolean indicating if the two info's are "equal" (i.e. basic
- *          weather attributes are the same)
- */
-
-gboolean
-weather_info_equal (WeatherInfo *info1, WeatherInfo *info2)
-{
-    g_return_val_if_fail (info1 != NULL, FALSE);
-    g_return_val_if_fail (info2 != NULL, FALSE);
-    g_return_val_if_fail (info1->valid,  FALSE);
-    g_return_val_if_fail (info2->valid,  FALSE);
-
-    /*
-     * This function is intended to be used to reduce notification spam.
-     * the idea is that unless there's a significant change to either
-     * temp, windspeed, or conditions, there's no need to issue a notification
-     * to the user.
-     */
-
-    if (info1->temp             != info2->temp)
-        return FALSE;
-    if (info1->wind             != info2->wind)
-        return FALSE;
-    if (info1->windspeed        != info2->windspeed)
-        return FALSE;
-    if (info1->cond.significant != info2->cond.significant)
-        return FALSE;
-    if (info1->cond.phenomenon  != info2->cond.phenomenon)
-        return FALSE;
-
-    return TRUE;
-}
